@@ -1,7 +1,6 @@
-/* -*- c-file-style: "linux";indent-tabs-mode:t -*- */
-/* Copyright (C) 2017 Jeremiah Orians
- * Copyright (C) 2017 Jan Nieuwenhuizen <janneke@gnu.org>
- * This file is part of mescc-tools
+/* Copyright (C) 2016 Jeremiah Orians
+ * Copyright (C) 2018 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+ * This file is part of mescc-tools.
  *
  * mescc-tools is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,12 +16,10 @@
  * along with mescc-tools.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#define max_string 4096
-//CONSTANT max_string 4096
-#define TRUE 1
-//CONSTANT TRUE 1
 #define FALSE 0
-//CONSTANT FALSE 0
+// CONSTANT FALSE 0
+#define TRUE 1
+// CONSTANT TRUE 1
 
 int in_set(int c, char* s)
 {
